﻿
local me = { name = "leotheras"}
local mod = thismod
mod[me.name] = me

--[[
Leotheras.lua

KTM Boss Mod for Leotheras in Serpentshrine Cavern.

----------------------------------------------------
		World of Warcraft Implementation
----------------------------------------------------

Leotheras starts bound by a few adds. When you kill them all he attacks, giving a yell. When Leotheras changes to demon form, he gives a yell and resets threat. When he changes to human form, he resets threat but has no yell or emote. When he casts whirlwind in human form, he aggros people randomly. When the whirlwind fades, he resets threat and resumes normal aggro.

----------------------------------------------------
			KTM Implementation
----------------------------------------------------

When we detect the Leotheras free yell, which is <mod.string.get("boss", "speech", "leotheraspull")>, we clear KTM and set the master target to the yeller.

The yell from demon form is given by <mod.string.get("boss", "speech", "leotheras")>. We check arg1 for a match on this from the "CHAT_MSG_MONSTER_YELL" event.

He casts whirlwind shortly after gaining human form, therefore we don't have to worry about the threat wipe when he changes to human.

Whirlwind fading uses the "AURAREMOVEDOTHER" pattern on the "CHAT_MSG_SPELL_AURA_GONE_OTHER" event. We check the name of the mob matches <mod.string.get("boss", "name", "leotheras")> and the spell matches <mod.string.get("boss", "spell", "whirlwind")>.

The implementation requires that (localisations of) the name of Leotheras and Whirlwind are exact, while the yell can be a partial match.

We send a generic threat wipe from the boss module ("shazzrahgate", 5 second cooldown), which requires 2 people to detect to be accepted by anyone who doesn't detect it.

----------------------------------------------------
			Interaction with other modules
----------------------------------------------------

We use the method <mod.boss.broadcastreset()> in the <boss> module.

We use these localisation keys, with enUS examples

	"boss" - "name" - "leotheras" = "Leotheras the Blind"
	"boss" - "speech" - "leotheraspull" = "Finally, my banishment ends!"
	"boss" - "speech" - "leotheras" = "Be gone, trifling elf"
	"boss" - "spell" - "whirlwind" = "Whirlwind"
	
----------------------------------------------------
				Credits
----------------------------------------------------

Thanks to Tandanu for the Leotheras module in Deadly Boss Mods from which i borrowed event names and strings. http://www.deadlybossmods.com/

----------------------------------------------------
				Debugging
----------------------------------------------------

To enable debugging of this module, uncomment these lines:

	me.mytraces = 
	{
		default = "info",
	}

Pull test codee:

	/script event = "CHAT_MSG_MONSTER_YELL"; arg1 = "Finally, my banishment ends!"; arg2 = "Leotheras the Blind"; klhtm.event.frameonevent();

Yell test code:

	/script event = "CHAT_MSG_MONSTER_YELL"; arg1 = "Be gone, trifling elf."; arg2 = "Leotheras the Blind"; klhtm.event.frameonevent();

Whirlwind test code:

	/script event = "CHAT_MSG_SPELL_AURA_GONE_OTHER"; arg1 = "Whirlwind fades from Leotheras the Blind."; klhtm.event.frameonevent();

]]

--[[
------------------------------------------------------------------------
			Services from Events.lua
------------------------------------------------------------------------
]]

me.myevents = { "CHAT_MSG_MONSTER_YELL" }

me.onevent = function()
	
	if arg2 == mod.string.get("boss", "name", "leotheras") then
		
		-- Leotheras attacks
		if string.find(arg1, mod.string.get("boss", "speech", "leotheraspull")) then
			
			-- debug
			if mod.trace.check("info", me, "pull") then
				mod.trace.print("leotheras attacks; sending reset and setting master target.")
			end
			
			mod.target.automastertarget(arg2)
			mod.boss.broadcastreset()
		
		-- Demon phase
		elseif string.find(arg1, mod.string.get("boss", "speech", "leotheras")) then
		
			-- debug
			if mod.trace.check("info", me, "demonphase") then
				mod.trace.print("leotheras into demon phase; sending reset.")
			end
			
			mod.boss.broadcastreset()
		end
	end
	
end

--[[
------------------------------------------------------------------------
			Services from Regex.lua
------------------------------------------------------------------------
]]

me.myparsers = 
{
	{ "buffgone", "AURAREMOVEDOTHER", "CHAT_MSG_SPELL_AURA_GONE_OTHER", }, -- %s fades from %s.
}

me.onparse = function(identifier, buffname, unitname) 
	
	if identifier == "buffgone" and unitname == mod.string.get("boss", "name", "leotheras") and buffname == mod.string.get("boss", "spell", "whirlwind") then
		
		if mod.trace.check("info", me, "demonphase") then
			mod.trace.print("leotheras finishes whirlwind; sending reset.")
		end
		
		mod.boss.broadcastreset()
	end
	
end
